@@ -3,6 +3,7 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 static gchar *appname = "dunstify";
 static gchar *summary = NULL;
@@ -13,6 +14,7 @@ static gchar **hint_strs = NULL;
 static gchar **action_strs = NULL;
 static gint timeout = NOTIFY_EXPIRES_DEFAULT;
 static gchar *icon = NULL;
+static gchar *raw_icon_path = NULL;
 static gboolean capabilities = false;
 static gboolean serverinfo = false;
 static gboolean printid = false;
@@ -28,6 +30,7 @@ static GOptionEntry entries[] =
     { "action", 'A', 0, G_OPTION_ARG_STRING_ARRAY, &action_strs, "Actions the user can invoke", "ACTION" },
     { "timeout", 't', 0, G_OPTION_ARG_INT, &timeout, "The time until the notification expires", "TIMEOUT" },
     { "icon",    'i', 0, G_OPTION_ARG_STRING, &icon, "An Icon that should be displayed with the notification", "ICON" },
+    { "raw_icon", 'I', 0, G_OPTION_ARG_STRING, &raw_icon_path, "Path to the icon to be sent as raw image data", "RAW_ICON"},
     { "capabilities",   'c', 0, G_OPTION_ARG_NONE, &capabilities, "Print the server capabilities and exit", NULL},
     { "serverinfo", 's', 0, G_OPTION_ARG_NONE, &serverinfo, "Print server information and exit", NULL},
     { "printid", 'p', 0, G_OPTION_ARG_NONE, &printid, "Print id, which can be used to update/replace this notification", NULL},
@@ -49,7 +52,7 @@ void print_capabilities(void)
     GList *caps = notify_get_server_caps();
     for (GList *iter = caps; iter; iter = iter->next) {
         if (strlen(iter->data) > 0) {
-            g_print("%s\n", iter->data);
+            g_print("%s\n", (char *)iter->data);
         }
     }
 }
@@ -171,7 +174,7 @@ int get_id(NotifyNotification *n)
     return kn->id;
 }
 
-int put_id(NotifyNotification *n, guint32 id)
+void put_id(NotifyNotification *n, guint32 id)
 {
     knickers *kn = n->priv;
 
@@ -199,7 +202,7 @@ void closed(NotifyNotification *n, gpointer foo)
 void add_action(NotifyNotification *n, char *str)
 {
     char *action = str;
-    char *label = strstr(str, ",");
+    char *label = strchr(str, ',');
 
     if (!label || *(label+1) == '\0') {
         g_printerr("Malformed action. Excpected \"action,label\", got \"%s\"", str);
@@ -215,14 +218,14 @@ void add_action(NotifyNotification *n, char *str)
 void add_hint(NotifyNotification *n, char *str)
 {
     char *type = str;
-    char *name = strstr(str, ":");
+    char *name = strchr(str, ':');
     if (!name || *(name+1) == '\0') {
         g_printerr("Malformed hint. Expected \"type:name:value\", got \"%s\"", str);
         return;
     }
     *name = '\0';
     name++;
-    char *value = strstr(name, ":");
+    char *value = strchr(name, ':');
     if (!value || *(value+1) == '\0') {
         g_printerr("Malformed hint. Expected \"type:name:value\", got \"%s\"", str);
         return;
@@ -240,7 +243,7 @@ void add_hint(NotifyNotification *n, char *str)
         gint h_byte = g_ascii_strtoull(value, NULL, 10);
         if (h_byte < 0 || h_byte > 0xFF)
             g_printerr("Not a byte: \"%s\"", value);
-        else 
+        else
             notify_notification_set_hint_byte(n, name, (guchar) h_byte);
     } else
         g_printerr("Malformed hint. Expected a type of int, double, string or byte, got %s\n", type);
@@ -249,8 +252,9 @@ void add_hint(NotifyNotification *n, char *str)
 
 int main(int argc, char *argv[])
 {
-
-    g_type_init();
+    #if !GLIB_CHECK_VERSION(2,35,0)
+        g_type_init();
+    #endif
     parse_commandline(argc, argv);
 
     if (!notify_init(appname)) {
@@ -265,6 +269,17 @@ int main(int argc, char *argv[])
 
     GError *err = NULL;
 
+    if (raw_icon_path) {
+            GdkPixbuf *raw_icon = gdk_pixbuf_new_from_file(raw_icon_path, &err);
+
+            if(err) {
+                g_printerr("Unable to get raw icon: %s\n", err->message);
+                die(1);
+            }
+
+            notify_notification_set_image_from_pixbuf(n, raw_icon);
+    }
+
     if (close_id > 0) {
         put_id(n, close_id);
         notify_notification_close(n, &err);
@@ -311,3 +326,5 @@ int main(int argc, char *argv[])
 
     die(0);
 }
+
+/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
